import json
import os
from datetime import datetime, timedelta

from notedrive.others.github.utils import (all_file_to_github,
                                           upload_data_to_github)
from noteread.legado.shelf.base import BookSource, BookSourceCorrect
from notetool.tool.pyrss import RSS2, RSSItem

book = BookSource(lanzou_fid=4147049)
book_correct = BookSourceCorrect(lanzou_fid=4147049)


def load():
    urls = [

        "https://cdn.jsdelivr.net/gh/bushixuanqi/book-source/%E5%85%A8%E7%BD%91%E9%80%9A%E7%94%A8.json",
        "https://cdn.jsdelivr.net/gh/bushixuanqi/book-source/%E4%B9%A6%E6%BA%90%E5%90%88%E9%9B%86.json",

        "https://gitee.com/z507525872/book-source/raw/master/32yousheng.json",
        "https://gitee.com/jia_to_hui/read-30-preferred-book-source/raw/master/bookSource.json",
        "https://guaner001125.coding.net/p/coding-code-guide/d/booksources/git/raw/master/sources/guaner.json",
        "https://haxc.coding.net/p/booksrc/d/booksrc/git/raw/master/Book3.0Source.json",
        "https://haxc.coding.net/p/booksrc/d/booksrc/git/raw/master/bookSource.json",

        "https://namofree.gitee.io/yuedu3/legado3_booksource_by_Namo.json",
        "https://no-mystery.gitee.io/shuyuan/%E5%85%A8%E7%BD%91%E9%80%9A%E7%94%A8.json",
        "https://olixina.coding.net/p/yuedu/d/source/git/raw/master/bookSource.json",
        "https://pbpobing.coding.net/p/yueduyuan/d/sy/git/raw/master/syhj.json",
        "https://pbpobing.coding.net/p/yueduyuan/d/sy/git/raw/master/yshj.json",
        "https://pbpobing.coding.net/p/yueduyuan/d/sy/git/raw/master/3syhj.json",
        "https://pbpobing.coding.net/p/yueduyuan/d/sy/git/raw/master/3yshj.json",

        "https://shuyuan.miaogongzi.site/shuyuan/1612930793.txt",
        "https://shuyuan.miaogongzi.site/shuyuan/1615385692.txt",
        "https://shuyuan.miaogongzi.net/shuyuan/1617076136.json",
        "https://shuyuan.miaogongzi.net/shuyuan/1617281456.json",

        "https://shuyuan.miaogongzi.net/shuyuan/1617406392.json",
        "https://shuyuan.miaogongzi.net/shuyuan/1619463469.txt",
        "https://shuyuan.miaogongzi.net/shuyuan/1619872864.json",
        "https://shuyuan.miaogongzi.net/shuyuan/1619911647.txt",
        "https://shuyuan.miaogongzi.net/shuyuan/1619967369.json",
        "https://shuyuan.miaogongzi.net/shuyuan/1620929243.json",
        "https://shuyuan.miaogongzi.net/shuyuan/1622744487.json",
        "https://shuyuan.miaogongzi.net/shuyuan/1623355431.json",

        "https://shuyuan.miaogongzi.net/shuyuan/1624832786.json",
        "https://shuyuan.miaogongzi.net/shuyuan/1622509629.json",
        "https://shuyuan.miaogongzi.net/shuyuan/1626191885.txt",
        "https://shuyuan.miaogongzi.net/shuyuan/1626711440.json"
        "https://shuyuan.miaogongzi.net/shuyuan/1626966311.json",
        "https://shuyuan.miaogongzi.net/shuyuan/1627802706.txt",
        "https://shuyuan.miaogongzi.net/shuyuan/1630342495.json",
        "https://shuyuan.miaogongzi.net/shuyuan/1631138099.json",
        "https://shuyuan.miaogongzi.net/shuyuan/1632140979.json",
        "https://shuyuan.miaogongzi.net/shuyuan/1633030418.json",
        "https://shuyuan.miaogongzi.net/shuyuan/1633826458.json",
        "https://shuyuan.miaogongzi.net/shuyuan/1630342495.json",

        "https://slaijie.gitee.io/legado/legado/BookSource210116.json",
        "https://tangguochaotian.coding.net/p/tangguoshuyuan1015/d/tangguo/git/raw/master/exportBookSource.json",
        "https://tianyuzhange.coding.net/p/booksource/d/shuyuan/git/raw/master/2.0shuyuan.json",

        "https://gitee.com/ch4nge/readbook/raw/master/booksource_fl",

        "https://gitee.com/ch4nge/readbook/raw/master/booksource200",
        "https://gitee.com/ch4nge/readbook/raw/master/booksource40",
        "https://gitee.com/YiJieSS/Yuedu/raw/master/bookSource.json",
    ]
    from noteread.legado.shelf.libs.core import load_from_url
    load_from_url(cate1='1', urls=urls, book=book)


def upload_version(cate1='1', repo='notechats/notefile', git_dir='notechats/noteread/data', split_num=1000, num=10):
    datas = book_correct.select(condition={'cate1': cate1})

    dd = datetime.now().strftime('%Y%m%d%H%M%S')
    for i in range(num):
        data = datas[i * split_num:(i + 1) * split_num]
        data = [json.loads(d['jsons']) for d in data]
        data = json.dumps(data)

        git_path = f"{git_dir}/legado-source-{dd}-{i}.json"
        upload_data_to_github(data, git_path, repo)


def data2xml(repo='notechats/notefile', git_dir='notechats/noteread'):
    files = all_file_to_github(os.path.join(git_dir, 'data'), repo_str=repo)
    files = sorted(files, key=lambda x: x, reverse=True)

    items = []
    for _file in files:
        res = os.path.basename(_file).split('.')[0].split('-')
        date = datetime.strptime(res[2], '%Y%m%d%H%M%S') + timedelta(seconds=int(res[3]))

        # github:
        link = f'http://www.baidu.com?wd=yuedu://rsssource/importonline?src=https://raw.githubusercontent.com/{repo}/master/file'
        # "gitee":
        # link = f'https://gitee.com/{repo}/raw/master/file'
        item = RSSItem(
            title=os.path.basename(_file),
            link=link,
            description=_file,
            # guid=PyRSS2Gen.Guid("https://github.com/notechats/notefile/tree/master/notechats/noteread"),
            pubDate=date)
        items.append(item)

    rss = RSS2(
        title="yuedu feed",
        link="https://github.com/notechats/notefile/tree/master/notechats/noteread",
        description="阅读的书源",
        lastBuildDate=datetime.now(),
        items=items)

    res = rss.to_xml(encoding='UTF-8')
    git_path = f"{git_dir}/legado-source.xml"

    upload_data_to_github(res, git_path, repo)


# load()
# correct_source(book, book_correct)
# book.db_save()
print(len(book.select_all()))
print(len(book_correct.select_all()))

# upload_version(cate1='1')
data2xml()



def _format_date(dt):
    """convert a datetime into an RFC 822 formatted date

    Input date must be in GMT.
    """
    # Looks like:
    #   Sat, 07 Sep 2002 00:00:01 GMT
    # Can't use strftime because that's locale dependent
    #
    # Isn't there a standard way to do this for Python?  The
    # rfc822 and email.Utils modules assume a timestamp.  The
    # following is based on the rfc822 module.
    return "%s, %02d %s %04d %02d:%02d:%02d GMT" % (
        ["Mon", "Tue", "Wed", "Thu", "Fri", "Sat", "Sun"][dt.weekday()],
        dt.day,
        ["Jan", "Feb", "Mar", "Apr", "May", "Jun",
         "Jul", "Aug", "Sep", "Oct", "Nov", "Dec"][dt.month - 1],
        dt.year, dt.hour, dt.minute, dt.second)


class Element:
    def __init__(self, *args, **kwargs):
        self.handler = None

    def publish(self, handler=None):
        self.handler = handler

    def write_element(self, name: str, value, d=None):
        d = d or {}
        if isinstance(value, str) or value is None:
            self.handler.startElement(name, d)
            if value is not None:
                self.handler.characters(value)
            self.handler.endElement(name)
        else:
            value.publish(self.handler)

    def write_opt_element(self, name: str, obj):
        if obj is None:
            return
        self.write_element(name, obj)


class IntElement(Element):
    element_attrs = {}

    def __init__(self, name, val, *args, **kwargs):
        super(IntElement, self).__init__(*args, **kwargs)
        self.name = name
        self.val = val

    def publish(self, handler=None):
        super(IntElement, self).publish(handler)
        self.handler.startElement(self.name, self.element_attrs)
        self.handler.characters(str(self.val))
        self.handler.endElement(self.name)


class DateElement(Element):
    def __init__(self, name, dt, *args, **kwargs):
        super(DateElement, self).__init__(*args, **kwargs)
        self.name = name
        self.dt = dt

    def publish(self, handler=None):
        super(DateElement, self).publish(handler)
        self.write_element(handler, self.name, _format_date(self.dt))


class Category(Element):
    def __init__(self, category, domain=None, *args, **kwargs):
        super(Category, self).__init__(*args, **kwargs)
        self.category = category
        self.domain = domain

    def publish(self, handler=None):
        super(Category, self).publish(handler)
        d = {}
        if self.domain is not None:
            d["domain"] = self.domain
        self.write_element("category", self.category, d)


class Cloud(Element):
    def __init__(self, domain, port, path, register_procedure, protocol, *args, **kwargs):
        super(Cloud, self).__init__(*args, **kwargs)
        self.domain = domain
        self.port = port
        self.path = path
        self.registerProcedure = register_procedure
        self.protocol = protocol

    def publish(self, handler=None):
        super(Cloud, self).publish(handler)
        self.write_element("cloud", None, {
            "domain": self.domain,
            "port": str(self.port),
            "path": self.path,
            "registerProcedure": self.registerProcedure,
            "protocol": self.protocol})


class Image(Element):
    """Publish a channel Image"""
    element_attrs = {}

    def __init__(self, url, title, link, width=None, height=None, description=None, *args, **kwargs):
        super(Image, self).__init__(*args, **kwargs)
        self.url = url
        self.title = title
        self.link = link
        self.width = width
        self.height = height
        self.description = description

    def publish(self, handler=None):
        super(Image, self).publish(handler)
        handler.startElement("image", self.element_attrs)

        self.write_element("url", self.url)
        self.write_element("title", self.title)
        self.write_element("link", self.link)

        width = self.width
        if isinstance(width, int):
            width = IntElement("width", width)
        self.write_opt_element("width", width)

        height = self.height
        if isinstance(height, int):
            height = IntElement("height", height)
        self.write_opt_element("height", height)

        self.write_opt_element("description", self.description)

        handler.endElement("image")


class Guid(Element):
    def __init__(self, guid, is_perma_link=1, *args, **kwargs):
        super(Guid, self).__init__(*args, **kwargs)
        self.guid = guid
        self.isPermaLink = is_perma_link

    def publish(self, handler=None):
        super(Guid, self).publish(handler)
        d = {}
        if self.isPermaLink:
            d["isPermaLink"] = "true"
        else:
            d["isPermaLink"] = "false"
        self.write_element("guid", self.guid, d)


class TextInput(Element):
    element_attrs = {}

    def __init__(self, title, description, name, link, *args, **kwargs):
        super(TextInput, self).__init__(*args, **kwargs)
        self.title = title
        self.description = description
        self.name = name
        self.link = link

    def publish(self, handler=None):
        super(TextInput, self).publish(handler)
        handler.startElement("textInput", self.element_attrs)
        self.write_element("title", self.title)
        self.write_element("description", self.description)
        self.write_element("name", self.name)
        self.write_element("link", self.link)
        handler.endElement("textInput")


class Enclosure(Element):
    def __init__(self, url, length, type, *args, **kwargs):
        super(Enclosure, self).__init__(*args, **kwargs)
        self.url = url
        self.length = length
        self.type = type

    def publish(self, handler=None):
        super(Enclosure, self).publish(handler)
        self.write_element("enclosure", None,
                           {"url": self.url,
                            "length": str(self.length),
                            "type": self.type,
                            })


class Source(Element):
    def __init__(self, name, url, *args, **kwargs):
        super(Source, self).__init__(*args, **kwargs)
        self.name = name
        self.url = url

    def publish(self, handler=None):
        super(Source, self).publish(handler)
        self.write_element("source", self.name, {"url": self.url})


class SkipHours(Element):
    element_attrs = {}

    def __init__(self, hours, *args, **kwargs):
        super(SkipHours, self).__init__(*args, **kwargs)
        self.hours = hours

    def publish(self, handler=None):
        super(SkipHours, self).publish(handler)
        if self.hours:
            handler.startElement("skipHours", self.element_attrs)
            for hour in self.hours:
                self.write_element("hour", str(hour))
            handler.endElement("skipHours")


class SkipDays(Element):
    element_attrs = {}

    def __init__(self, days, *args, **kwargs):
        super(SkipDays, self).__init__(*args, **kwargs)
        self.days = days

    def publish(self, handler=None):
        super(SkipDays, self).publish(handler)
        if self.days:
            handler.startElement("skipDays", self.element_attrs)
            for day in self.days:
                self.write_element("day", day)
            handler.endElement("skipDays")
